use futures::TryStreamExt;
use k8s_openapi::api::core::v1::Secret;
use kube::{Api, Client};
use kube::runtime::{watcher, WatchStreamExt};
use minio_operator::constants::{SECRET_MINIO_INSTANCE_ACCESS_KEY, SECRET_MINIO_INSTANCE_SECRET_KEY};
use minio_operator::crd::{MinioBucket, MinioInstance};
use minio_operator::minio::MinioService;
use minio_operator::secrets::read_secret_str;

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));

    let client = Client::try_default().await?;

    let buckets: Api<MinioBucket> = Api::default_namespaced(client.clone());

    // Listen for events / buckets creation or update (deletion is not supported)
    let wc = watcher::Config::default();
    let bw = watcher(buckets, wc).applied_objects();
    futures::pin_mut!(bw);

    while let Some(b) = bw.try_next().await? {
        if let Err(e) = apply_bucket(&b, &client).await {
            log::error!("Failed to apply desired configuration for applied bucket {} : {}", b.spec.name, e)
        }
    }

    Ok(())
}


/// Make sure a bucket is compliant with a desired configuration
async fn apply_bucket(b: &MinioBucket, client: &Client) -> anyhow::Result<()> {
    log::info!("Apply configuration for bucket {}", b.spec.name);

    // Get instance information
    let instances: Api<MinioInstance> = Api::default_namespaced(client.clone());
    let instance = instances.get(&b.spec.instance).await?;

    // Get instance configuration
    let secrets: Api<Secret> = Api::default_namespaced(client.clone());
    let instance_secret = secrets.get(&instance.spec.credentials).await?;
    let service = MinioService {
        hostname: instance.spec.endpoint,
        access_key: read_secret_str(&instance_secret, SECRET_MINIO_INSTANCE_ACCESS_KEY)?,
        secret_key: read_secret_str(&instance_secret, SECRET_MINIO_INSTANCE_SECRET_KEY)?,
    };
    
    println!("{:?}", service);

    Ok(())
}